@@ -1,6 +1,5 @@
 import sys
 import typing
-from typing import Optional
 
 import asn1tools
 import wx
@@ -10,25 +9,26 @@ import asn1editor
 from asn1editor.ASN1SpecHandler import ASN1SpecHandler
 from asn1editor.Plugin import Plugin
 from asn1editor.PluginInterface import PluginInterface
-from asn1editor.wxPython import Settings as Settings
+from asn1editor.wxPython import Environment
 from asn1editor.wxPython import WxPythonViewFactory
 from asn1editor.wxPython.FilePickerHandler import FilePickerHandler
 from asn1editor.wxPython.Resources import resource_path
 
 
 class MainWindow(wx.Frame, PluginInterface):
-    def __init__(self, plugin: Optional[Plugin] = None, title='ASN.1 editor'):
-        super(MainWindow, self).__init__(None, title=f'{title} {asn1editor.__version__}', size=(500, 800))
+    def __init__(self, plugins: typing.Optional[typing.List[Plugin]] = None, title=f'ASN.1 editor {asn1editor.__version__}'):
+        super(MainWindow, self).__init__(None, title=title, size=(500, 800))
 
-        self.__plugin = plugin
-        if self.__plugin is not None:
-            self.__plugin.connect(self)
+        Environment.load()
 
-        Settings.load()
+        self.__plugins = plugins
+        if self.__plugins is not None:
+            for plugin in self.__plugins:
+                plugin.connect(self)
 
-        self.SetSize(wx.Size(Settings.settings.get('size', (500, 800))))
-        self.Maximize(Settings.settings.get('maximized', True))
-        self.SetPosition(wx.Point(Settings.settings.get('position', (0, 0))))
+        self.SetSize(wx.Size(Environment.settings.get('size', (500, 800))))
+        self.Maximize(Environment.settings.get('maximized', True))
+        self.SetPosition(wx.Point(Environment.settings.get('position', (0, 0))))
 
         self.__main_panel = wx.ScrolledWindow(self, style=wx.HSCROLL | wx.VSCROLL)
         self.__main_panel.SetScrollbars(15, 15, 50, 50)
@@ -47,6 +47,8 @@ class MainWindow(wx.Frame, PluginInterface):
         self.__type_name = None
         self.__file_name = None
 
+        self.__progress_window: typing.Optional[wx.ProgressDialog] = None
+
         self.bind_events()
 
         # noinspection SpellCheckingInspection
@@ -77,20 +79,22 @@ class MainWindow(wx.Frame, PluginInterface):
         self.__exit_item.SetBitmap(image.ConvertToBitmap(width=16, height=16))
         menu_bar.Append(file_menu, '&File')
 
-        if self.__plugin is not None:
-            plugin_menu = wx.Menu()
-            menus = self.__plugin.get_menus()
-            for i, menu in enumerate(menus):
-                if not len(menu[0]):
-                    plugin_menu.AppendSeparator()
-                else:
-                    menu_item: wx.MenuItem = plugin_menu.Append(i, menu[0])
-                    if menu[1] is not None:
-                        self.Bind(wx.EVT_MENU, self.__plugin_menu_event, menu_item)
-                    else:
-                        menu_item.Enable(False)
+        if self.__plugins is not None:
 
-            menu_bar.Append(plugin_menu, self.__plugin.get_name())
+            for plugin_index, plugin in enumerate(self.__plugins):
+                plugin_menu = wx.Menu()
+                menus = plugin.get_menus()
+                for i, menu in enumerate(menus):
+                    if not len(menu[0]):
+                        plugin_menu.AppendSeparator()
+                    else:
+                        menu_item: wx.MenuItem = plugin_menu.Append(plugin_index * 1000 + i, menu[0])
+                        if menu[1] is not None:
+                            self.Bind(wx.EVT_MENU, self.__plugin_menu_event, menu_item)
+                        else:
+                            menu_item.Enable(False)
+
+                menu_bar.Append(plugin_menu, plugin.get_name())
 
         help_menu = wx.Menu()
         about_item = help_menu.Append(wx.ID_ABOUT, 'About')
@@ -100,7 +104,8 @@ class MainWindow(wx.Frame, PluginInterface):
         self.SetMenuBar(menu_bar)
 
     def __plugin_menu_event(self, e):
-        self.__plugin.get_menus()[e.GetId()][1]()
+        plugin_index = e.GetId() // 1000
+        self.__plugins[plugin_index].get_menus()[e.GetId() % 1000][1]()
 
     def bind_events(self):
         def schema_dialog_constructor() -> wx.FileDialog:
@@ -129,7 +134,7 @@ class MainWindow(wx.Frame, PluginInterface):
 
         self.Bind(wx.EVT_CLOSE, self.close, self)
 
-    def load_spec(self, file_name: str, type_name: Optional[str] = None):
+    def load_spec(self, file_name: str, type_name: typing.Optional[str] = None):
         # Spec file loaded, compile it to show a selection of type names
         if not self.__asn1_handler or file_name not in self.__asn1_handler.get_filename():
             self.__asn1_handler = ASN1SpecHandler(file_name)
@@ -210,14 +215,21 @@ class MainWindow(wx.Frame, PluginInterface):
     def get_spec(self, codec: str):
         return self.__asn1_handler.get_compiled(codec)
 
-    def text_entry(self, message: str) -> typing.Optional[str]:
+    def text_entry(self, message: str, default: typing.Optional[str] = None) -> typing.Optional[str]:
         with wx.TextEntryDialog(self, message) as text_dialog:
+            if default is not None:
+                text_dialog.SetValue(default)
             if text_dialog.ShowModal() == wx.ID_CANCEL:
                 return
             return text_dialog.GetValue()
 
-    def choice_entry(self, message: str, choices: typing.List[str]) -> typing.Optional[str]:
-        with wx.SingleChoiceDialog(self, message, '', choices=choices) as choice_dialog:
+    def choice_entry(self, message: str, caption: str, choices: typing.List[str], default: typing.Optional[str] = None) -> typing.Optional[str]:
+        with wx.SingleChoiceDialog(self, message, caption, choices=choices) as choice_dialog:
+            if default is not None:
+                try:
+                    choice_dialog.SetSelection(choices.index(default))
+                except ValueError:
+                    pass
             if choice_dialog.ShowModal() == wx.ID_CANCEL:
                 return
             return choice_dialog.GetStringSelection()
@@ -225,13 +237,32 @@ class MainWindow(wx.Frame, PluginInterface):
     def show_status(self, message: str):
         self._status_bar.SetStatusText(message)
 
-    def show_message(self, message: str, message_type: PluginInterface.MessageType):
-        style = wx.OK | wx.CENTER | {PluginInterface.MessageType.WARNING: wx.ICON_WARNING, PluginInterface.MessageType.INFO: wx.ICON_INFORMATION,
-                                     PluginInterface.MessageType.ERROR: wx.ICON_ERROR}[message_type]
-        wx.MessageBox(message, 'Plugin', style=style)
+    def show_message(self, message: str, caption: str, message_type: PluginInterface.MessageType) -> bool:
+        style = wx.CENTER | {PluginInterface.MessageType.WARNING: wx.OK | wx.ICON_WARNING,
+                             PluginInterface.MessageType.INFO: wx.OK | wx.ICON_INFORMATION,
+                             PluginInterface.MessageType.ERROR: wx.OK | wx.ICON_ERROR,
+                             PluginInterface.MessageType.QUESTION: wx.YES_NO | wx.ICON_QUESTION}[message_type]
+        ret = wx.MessageBox(message, caption, style=style)
+        return True if message_type != PluginInterface.MessageType.QUESTION else wx.YES == ret
+
+    def show_progress(self, message: str, caption: str, max_progress: typing.Optional[int] = None):
+        self.__progress_window = wx.ProgressDialog(caption, message, maximum=max_progress if max_progress else 100, style=wx.PD_APP_MODAL | wx.PD_CAN_ABORT)
+
+    def update_progress(self, message: typing.Optional[str] = None, close: bool = False, progress: typing.Optional[int] = None) -> bool:
+        running = False
+        if self.__progress_window:
+            if progress is not None:
+                running = self.__progress_window.Update(progress, newmsg=message)[0]
+            else:
+                running = self.__progress_window.Pulse()[0]
+        if close:
+            self.__progress_window.Close()
+            self.__progress_window = None
+
+        return running
 
     def get_settings(self) -> dict:
-        return Settings.settings.setdefault('Plugin', {})
+        return Environment.settings.setdefault('Plugin', {})
 
     def __exception_handler(self, exc_type, value, trace):
         import traceback
@@ -242,12 +273,7 @@ class MainWindow(wx.Frame, PluginInterface):
         exception_str = f'{message}\n\n{trace}'
         wx.MessageBox(message, 'Error', wx.OK | wx.ICON_ERROR, parent=self)
         self._status_bar.SetStatusText(f'Error: {value}')
-        try:
-            with open('error_log.txt', 'a+') as f:
-                import datetime
-                f.write(f'{datetime.datetime.now()}: {exception_str}\n\n')
-        finally:
-            pass
+        Environment.log_error(exception_str)
 
     # noinspection PyUnusedLocal
     def __about_item_event(self, e):
@@ -271,10 +297,10 @@ https://github.com/eerimoq/asn1tools
 
     # noinspection PyUnusedLocal
     def close(self, e: wx.Event):
-        Settings.settings['size'] = (self.GetSize().Get())
-        Settings.settings['maximized'] = self.IsMaximized()
-        Settings.settings['position'] = self.GetPosition().Get()
+        Environment.settings['size'] = (self.GetSize().Get())
+        Environment.settings['maximized'] = self.IsMaximized()
+        Environment.settings['position'] = self.GetPosition().Get()
 
-        Settings.save()
+        Environment.save()
 
         self.Destroy()
