@@ -1,8 +1,9 @@
 import os
+import sys
 
 
 def resource_path(relative_path: str) -> str:
     """ Get absolute path to resource required for PyInstaller """
     # noinspection SpellCheckingInspection
-    base_path = os.environ.get("_MEIPASS2", os.path.dirname(os.path.abspath(__file__)))
+    base_path = getattr(sys, '_MEIPASS', os.path.dirname(os.path.abspath(__file__)))
     return os.path.join(base_path, relative_path)
